import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import javax.microedition.io.*;
import com.nttdocomo.io.*;
import com.nttdocomo.ui.*;

public class MoonCanvas extends Canvas implements Runnable {
	private IApplication application = null;
	private int last_age = -1;
	private boolean now_loading = false;
	private MediaImage mimage = null;
	private Image moon = null;
	private ShortTimer timer = null;
	private String to2Digit(int a) {
		String tmpstr = "00" + a;
		return tmpstr.substring(tmpstr.length() - 2);
	}
	private synchronized int getLastAge() throws IOException {
		DataInputStream in = Connector.openDataInputStream("scratchpad:///0");
		int age = in.read() - 1;
		in.close();
		return age;
	}
	private synchronized void loadImageToScratchPad(int age)
		throws IOException {
		HttpConnection conn = (HttpConnection) Connector.open(
			application.getSourceURL() + "moon" + to2Digit(age) + ".gif"
			, Connector.READ, true);
		conn.setRequestMethod(HttpConnection.GET);
		conn.connect();
		InputStream in = conn.openInputStream();
		DataOutputStream out
			= Connector.openDataOutputStream("scratchpad:///0");
		out.write(age + 1);
		byte[] buf = new byte[1024];
		int len = 0;
		while ((len = in.read(buf, 0, buf.length)) >= 0)
			out.write(buf, 0, len);
		out.close();
		in.close();
		conn.close();
	}
	private void loadImage(int age) {
		try {
			if (age != getLastAge())
				loadImageToScratchPad(age);
			if (moon != null) {
				moon.dispose();
				moon = null;
				mimage.unuse();
				mimage.dispose();
				mimage = null;
				System.gc();
			}
			mimage = MediaManager.getImage("scratchpad:///0;pos=1");
			mimage.use();
			moon = mimage.getImage();
		} catch (Throwable e) {
			Dialog d = new Dialog(Dialog.DIALOG_ERROR, "Error");
/*			if (e instanceof UIException)
				d.setText(e.toString() + ((UIException) e).getStatus());
			else
				d.setText(e.toString()); */
			d.setText("̲ޤ.");
			d.show();
		}
	}
	public void run() {
		loadImage(last_age);
		now_loading = false;
	}
	public MoonCanvas(IApplication application) {
		this.application = application;
		setSoftLabel(SOFT_KEY_1, "Quit");
	}
	public void start() {
		if (timer != null)
			timer.dispose();
		timer = ShortTimer.getShortTimer(this, 0, 1000, true); // ID = 0
		timer.start();
	}
	public void paint(Graphics g) {
		Calendar c = Calendar.getInstance();
		c.setTime(new Date());
		int moonage = ((c.get(Calendar.YEAR) - 1740) * 210 / 19 - 1
			+ c.get(Calendar.MONTH) + c.get(Calendar.DAY_OF_MONTH)) % 30;
		String datestr = "" + c.get(Calendar.YEAR) + "/"
			+ to2Digit(c.get(Calendar.MONTH) + 1) + "/"
			+ to2Digit(c.get(Calendar.DAY_OF_MONTH)) + " "
			+ to2Digit(c.get(Calendar.HOUR_OF_DAY)) + ":"
			+ to2Digit(c.get(Calendar.MINUTE)) + ":"
			+ to2Digit(c.get(Calendar.SECOND));
		g.lock();
		g.setColor(g.getColorOfName(g.BLACK));
		g.fillRect(0, 0, getWidth(), getHeight());
		Font font = Font.getFont(Font.FACE_PROPORTIONAL | Font.STYLE_PLAIN
			| Font.SIZE_SMALL);
		g.setFont(font);
		if (moonage != last_age) {
			now_loading = true;
			last_age = moonage;
			new Thread(this).start();
		}
		if (moon != null)
			g.drawImage(moon, getWidth() / 2 - moon.getWidth() / 2
				, (getHeight() - font.getHeight()) / 2 - moon.getHeight() / 2);
		if (now_loading) {
			String waiting = "Now loading...";
			g.setColor(g.getColorOfName(g.GRAY));
			g.drawString(waiting, getWidth() - font.stringWidth(waiting) + 1
				, getHeight() - font.getHeight() * 2 + font.getAscent() + 1);
			g.setColor(g.getColorOfName(g.WHITE));
			g.drawString(waiting, getWidth() - font.stringWidth(waiting)
				, getHeight() - font.getHeight() * 2 + font.getAscent());
		} 
		g.setColor(g.getColorOfName(g.GRAY));
		g.drawString(datestr, getWidth() - font.stringWidth(datestr) + 1
			, getHeight() - font.getHeight() + font.getAscent() + 1);
		g.setColor(g.getColorOfName(g.WHITE));
		g.drawString(datestr, getWidth() - font.stringWidth(datestr)
			, getHeight() - font.getHeight() + font.getAscent());
		g.unlock(false);
	}
	public void processEvent(int type, int param) {
		if ((type == Display.TIMER_EXPIRED_EVENT) && (param == 0))
			repaint();
		if ((type == Display.KEY_PRESSED_EVENT)
			&& (param == Display.KEY_SOFT1)) {
			if (timer != null)
				timer.dispose();
			application.terminate();
		}
	}
}
